#
#    Torinj experimental proxy used for experiment #1
#
#    Copyright (C) 2009 Alexandre Dulaunoy <a@foo.be>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

use HTTP::Proxy qw(:log);
use HTTP::Proxy::BodyFilter::simple;
use Data::GUID;

#use DBI;
#use DBD::SQLite;
use Sys::Syslog qw(:DEFAULT setlogsock);

# Autoflush added
$| = 1;

# config
# config : loggingtype if not specified SQLite
my $loggingtype = "syslog";

# alternate initialisation
my $proxy = HTTP::Proxy->new;
$proxy->host("0.0.0.0");
$proxy->port(3129);
$proxy->x_forwarded_for(false);
$proxy->via("");
$proxy->max_clients(250);

my $reqcount = 0;

my $filter = HTTP::Proxy::BodyFilter::simple->new(
    sub {
        my ( $self, $dataref, $message, $protocol, $buffer ) = @_;

        my $uid = addRequest(
            $message->request->uri(),
            $message->request->header("User-Agent"),
            $message->request->as_string
        );
        $reqcount++;
        if ( ( $reqcount % 100 ) == 0 ) {
            print $reqcount. " requests processed\n";
        }
        ${ $_[1] } =~
s/\<\/body\>/\<img src=\"http:\/\/$uid.exp.quuxlabs.com\/p.png\" \/>\<img src=\"http:\/\/f.exp.quuxlabs.com\/p.png\" \/>\<\/body\>/ig;
    }
);

$proxy->push_filter( response => $filter );

# we keep a full trace for the experiment along with the full frame pcap files 

my $logfileloc = ">>proxy.log";
open my $log, $logfileloc or die "Can't open $logfileloc for reading: $!";
$proxy->logfh($log);
$proxy->logmask( STATUS | HEADERS );
$proxy->start;

sub generateUID {
    my $guid = Data::GUID->new;
    return substr( $guid->as_hex, 2 );
}

sub addRequest {
    my ( $uri, $useragent, $fullheaders ) = @_;
    my $uid = generateUID();
    if ( $loggingtype eq "syslog" ) {
        my $msg = $uri . "\t" . $useragent . "\t" . $fullheaders;
        logmsg( "info", $msg );
    }
    else {
        dbAddRequest( time(), $uid, $uri, $useragent, $fullheaders );
    }
    return $uid;
}

#
#CREATE TABLE tRequest(
#        i INTEGER PRIMARY KEY AUTOINCREMENT,
#        t TIMESTAMP,
#        uid TEXT,
#        uri TEXT,
#        uagent TEXT,
#        full BLOB
#);

sub dbAddRequest {
    my ( $t, $uid, $uri, $uagent, $full ) = @_;

    my $dbargs = {
        AutoCommit => 1,
        PrintError => 1
    };

    my $dbh =
      DBI->connect( "dbi:SQLite:dbname=./db/proxy.db", "", "", $dbargs );

    $sth = $dbh->prepare(
        "INSERT INTO tRequest(i,t,uid,uri,uagent,full) VALUES (NULL,?,?,?,?,?)"
    );
    $sth->execute( $t, $uid, $uri, $uagent, $full );

    if ( $dbh->err() ) { die "$DBI::errstr\n"; }

    $dbh->commit();
    $dbh->disconnect();
}

sub logmsg {
    my ( $priority, $msg ) = @_;
    return 0 unless ( $priority =~ /info|err|debug/ );

    setlogsock('unix');
    openlog( $0, 'pid,cons', 'user' );
    syslog( $priority, $msg );
    closelog();
    return 1;
}

